/*
Module : FTPTransferDlg.h
Purpose: Defines the interface for an MFC dialog which performs FTP uploads and downloads similiar to the
         old Internet Explorer download dialog
Created: PJN / 3-01-2000

Copyright (c) 2000 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

All rights reserved.

*/


//////////////////// Macros ///////////////////////////////////////////////////

#pragma once

#ifndef __FTPTRANSFER_H__
#define __FTPTRANSFER_H__

#ifndef CFTPTRANSFERDLG_EXT_CLASS
#define CFTPTRANSFERDLG_EXT_CLASS
#endif //#ifndef CFTPTRANSFERDLG_EXT_CLASS


//////////////////// Includes /////////////////////////////////////////////////

#ifndef __FTPTRANSFERER_H__
#include "FTPTransferer.h"
#endif //#ifndef __FTPTRANSFERER_H__


//////////////////// Classes //////////////////////////////////////////////////

class CFTPTRANSFERDLG_EXT_CLASS CFTPTransferDlg : public CDialog,
                                                  public CFTPTransferer
{
public:
//Constructors / Destructors
  CFTPTransferDlg(_In_opt_ CWnd* pParent = nullptr);
  CFTPTransferDlg(const CFTPTransferDlg&) = delete;
  CFTPTransferDlg(CFTPTransferDlg&&) = delete;
  ~CFTPTransferDlg() = default; //NOLINT(modernize-use-override)

//Methods
  CFTPTransferDlg& operator=(const CFTPTransferDlg&) = delete;
  CFTPTransferDlg& operator=(CFTPTransferDlg&&) = delete;

protected:
//Enums
  enum class UIUpdate
  {
    None,
    Percentage,
    TimeLeft,
    Rate,
    Status
  };
  enum { IDD = IDD_FTPTRANSFER };

//Member variables
  CStatic m_ctrlStatus;
  CStatic m_ctrlTransferRate;
  CStatic m_ctrlTimeLeft;
  CProgressCtrl m_ctrlProgress;
  CStatic m_ctrlFileStatus;
  CAnimateCtrl m_ctrlAnimate;
  CString m_sError;
  bool m_bSafeToClose;
  ULONGLONG m_nLastTotalBytes;
  DWORD m_dwLastPercentage;
  ULONGLONG m_nLastTicks;
  ULONGLONG m_nStartTicks;
  CWinThread* m_pThread;
  CString m_sUIUpdateText;
  UIUpdate m_UIUpdate;
  ATL::CComAutoCriticalSection m_csUIUpdateText;

//Methods
  static UINT _TransferThread(_In_ LPVOID pParam);
  void DoDataExchange(CDataExchange* pDX) override;
  BOOL OnInitDialog() override;
  virtual void TransferThread();
  virtual void SetPercentage(_In_ int nPercentage);
  virtual void SetTimeLeft(_In_ DWORD dwSecondsLeft, _In_ ULONGLONG nBytesRead, _In_ ULONGLONG nFileSize);
  void SetStatus(_In_z_ LPCTSTR pszStatus) override;
  virtual void SetTransferRate(_In_ double KbPerSecond);
  void OnProgress(_In_ ULONGLONG nBytesRead, _In_ bool bGotFileSize, _In_ ULONGLONG nTotalBytesToRead) override;
  void HandleError(_In_ UINT nID, _In_ DWORD dwError) override;
  void OnWininetStatusCallBack(_In_ HINTERNET hInternet, _In_ DWORD dwInternetStatus, _In_ LPVOID lpvStatusInformation, _In_ DWORD dwStatusInformationLength) override;
  void OnCancel() override;

//Message handlers
  afx_msg void OnDestroy();
  afx_msg void OnClose();
  afx_msg LRESULT OnThreadFinished(WPARAM wParam, LPARAM lParam);
  afx_msg LRESULT OnUIUpdate(WPARAM wParam, LPARAM lParam);

  DECLARE_MESSAGE_MAP()
  DECLARE_DYNAMIC(CFTPTransferDlg);
};

#endif //#ifndef __FTPTRANSFER_H__
